@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: client.c,v 1.39 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: client.c,v 1.42 2006/03/30 23:36:49 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -27,7 +27,10 @@ MA 02110-1301 USA
 #include <errno.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include <string.h>
+#include <sys/un.h>
 #include <assert.h>
 
 #include "conf.h"
@@ -546,37 +549,87 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
   struct addrinfo hints;
   struct addrinfo *ai_res;
   struct addrinfo *ai;
+  struct sockaddr_un un_sock_addr;
+  struct stat stat_buf;
+  int gai_lookup; /* boolean */
   int result;
 
   memset(&hints, 0, sizeof(hints));
 
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_protocol = IPPROTO_TCP;
-  hints.ai_flags = AI_CANONNAME;
-  hints.ai_family = PF_UNSPEC;
-  hints.ai_addrlen = 0;
-  hints.ai_addr = 0;
-  hints.ai_canonname = 0;
+  if (nepim_global.search_unix_socket) {
+    gai_lookup = stat(hostname, &stat_buf);
+    if (gai_lookup)
+      fprintf(stderr, "not a UNIX domain path: %s: errno=%d: %s\n",
+	      hostname, errno, strerror(errno));
+  }
+  else
+    gai_lookup = -1;
 
-  fprintf(stderr, 
-          "TCP socket solving %s,%s\n",
-          hostname, portname);
+  if (gai_lookup) {
+    /*
+      Lookup hostname
+     */
 
-  result = getaddrinfo(hostname, portname,
-                       &hints, &ai_res);
-  if (result) {
-    fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
-            __PRETTY_FUNCTION__, hostname, portname,
-            gai_strerror(result));
-    return;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = IPPROTO_TCP;
+    hints.ai_flags = AI_CANONNAME;
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_addrlen = 0;
+    hints.ai_addr = 0;
+    hints.ai_canonname = 0;
+
+    fprintf(stderr, 
+	    "TCP socket solving %s,%s\n",
+	    hostname, portname);
+
+    result = getaddrinfo(hostname, portname,
+			 &hints, &ai_res);
+    if (result) {
+      fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
+	      __PRETTY_FUNCTION__, hostname, portname,
+	      gai_strerror(result));
+      return;
+    }
+  }
+  else {
+    /*
+      Unix socket
+     */
+
+    int len = strlen(hostname);
+    if (len >= (int) sizeof(un_sock_addr.sun_path)) {
+      fprintf(stderr, "%s %s: unix socket path overly long: len=%d > max=%d\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      len, sizeof(un_sock_addr.sun_path) - 1);
+      return;
+    }
+
+    memset(&un_sock_addr, 0, sizeof(un_sock_addr));
+
+    un_sock_addr.sun_family = AF_UNIX;
+    strcpy(un_sock_addr.sun_path, hostname);
+
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = 0;
+    hints.ai_family = AF_UNIX;
+    hints.ai_addrlen = sizeof(un_sock_addr);
+    hints.ai_addr = (struct sockaddr *) &un_sock_addr;
+    hints.ai_canonname = 0;
+
+    ai_res = &hints;
   }
 
   for (ai = ai_res; ai; ai = ai->ai_next) {
     int sd;
 
-    fprintf(stderr, 
-            "TCP socket trying %s,%s\n",
-            hostname, portname);
+    if (ai->ai_family == PF_UNIX)
+      fprintf(stderr, 
+	      "UNIX stream socket trying path=%s\n",
+	      hostname);
+    else
+      fprintf(stderr, 
+	      "TCP socket trying %s,%s\n",
+	      hostname, portname);
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
       continue;
@@ -590,21 +643,34 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
                                      nepim_global.pmtu_mode,
                                      nepim_global.ttl,
                                      nepim_global.win_recv,
-                                     nepim_global.win_send);
+                                     nepim_global.win_send,
+				     gai_lookup /* nodelay */,
+				     nepim_global.tos);
     if (sd < 0) {
-      fprintf(stderr, 
-              "could not connect TCP socket to %s,%s: %d\n",
-              hostname, portname, sd);
+      if (ai->ai_family == AF_UNIX)
+	fprintf(stderr, 
+		"could not connect UNIX stream socket to path=%s: %d\n",
+		hostname, sd);
+      else
+	fprintf(stderr, 
+		"could not connect TCP socket to %s,%s: %d\n",
+		hostname, portname, sd);
+
       continue;
     }
 
-    fprintf(stderr, 
-            "%d: TCP socket connected to: "
-            "host=%s,%s len=%d family=%d type=%d proto=%d\n",
-            sd, hostname, portname, 
-            ai->ai_addrlen, ai->ai_family, 
-            ai->ai_socktype, ai->ai_protocol);
-
+    if (ai->ai_family == AF_UNIX)
+      fprintf(stderr, 
+	      "%d: UNIX stream socket connected to: path=%s\n",
+	      sd, hostname); 
+    else
+      fprintf(stderr, 
+	      "%d: TCP socket connected to: "
+	      "host=%s,%s len=%d family=%d type=%d proto=%d\n",
+	      sd, hostname, portname, 
+	      ai->ai_addrlen, ai->ai_family, 
+	      ai->ai_socktype, ai->ai_protocol);
+    
     {
       nepim_greet_t opt;
 
@@ -629,7 +695,8 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
     break;
   }
 
-  freeaddrinfo(ai_res);
+  if (gai_lookup)
+    freeaddrinfo(ai_res);
 }
 
 static void parse_hosts(const char *host_list)
@@ -670,13 +737,14 @@ static void spawn_tcp_clients(const char *host_list)
 
   /* activate sockets */
   for (i = 0; i < pipes.array.capacity; ++i) {
-    char peer[100];
+    char peer[500];
     nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, i);
     if (!pipe)
       continue;
 
     nepim_sock_dump_addr(peer, sizeof(peer),
-                         (const struct sockaddr *) &pipe->session.remote);
+                         (const struct sockaddr *) &pipe->session.remote,
+			 pipe->session.remote_len);
 
     if (greet(i)) {
       fprintf(stderr,
