@@ -44,6 +44,8 @@ nepim_global_t nepim_global = {
   -1,      /* pmtu_mode */
   0,       /* bind_list */
   0,       /* join_list */
+  0,       /* unix_list */
+  0,       /* unix_dgram_list */
   -1,      /* ttl */
   -1,      /* mcast_ttl */
   500000,  /* udp_greet_delay (usec) */
@@ -66,6 +68,9 @@ nepim_global_t nepim_global = {
   0,       /* udp_exp_loss */
   0,       /* udp_exp_dup */
   0,       /* tcpwrap */
+  -1,      /* search_unix_socket (boolean) */
+  -1,      /* soft_error_verbose (boolean) */
+  -1,      /* ip tos (default: don't change) */
 
   0, /* oop source */
   0  /* source interface */
