@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.34 2006/03/10 18:56:51 evertonm Exp $
+  $Id: main.c,v 1.39 2006/03/31 19:14:34 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -96,6 +96,8 @@ static void usage(FILE *out, const char *prog_name)
           "  -k password    server authentication secret\n"
           "  -6             disable IPv6\n"
           "  -4             disable IPv4\n"
+	  "  -q             quietly hide soft read/write errors\n"
+	  "  -y tos         ip tos (defaults to %d, tos=-1 means don't change)\n"
           "  -x [win]       experimental UDP reception statistics (win=%d)\n"
           "  -L loss        experimental UDP sending loss\n"
           "  -D dup         experimental UDP sending duplication\n"
@@ -121,8 +123,13 @@ static void usage(FILE *out, const char *prog_name)
           "  -f [lib-name]  enable TCP wrapper filtering\n"
           "  -b addr-list   bind to specific addresses (defaults to any)\n"
           "  -j addr-list   join multicast groups (defaults to empty)\n"
-          "                 addr-list: addr_1[,port_1][/.../addr_n[,port_n]]\n"
-          "                 example:   10.0.0.1/192.168.0.1,5000/::1\n"
+	  "  -U path-list   bind UNIX stream sockets to specified addresses\n"
+#if 0
+	  "  -G path-list   bind UNIX datagram sockets to specified addresses\n"
+#endif
+          "                 list: addr_1[,port_1][... addr_n[,port_n]]\n"
+          "                 examples:  -b '10.0.0.1 192.168.0.1,5000 ::1'\n"
+	  "                            -U '/tmp/sock1 /tmp/sock2'\n"
           "  -l backlog     TCP listen(2) backlog (defaults to %d)\n"
           "  -T mcast-ttl   TTL for multicast packets (defaults to system)\n"
           ,
@@ -133,6 +140,7 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.tcp_read_size,
           nepim_global.udp_write_size,
           nepim_global.udp_read_size,
+	  nepim_global.tos,
           nepim_global.udp_win_max,
           nepim_global.pipes,
           bit_rate,
@@ -193,6 +201,22 @@ int main(int argc, const char *argv[])
   for (i = 1; i < argc; ++i) {
     const char *arg = argv[i];
 
+    if (!strcmp(arg, "-y")) {
+      ++i;
+      if (i >= argc) {
+        fprintf(stderr, "%s: missing tos\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      nepim_global.tos = atoi(argv[i]);
+      continue;
+    }
+
+    if (!strcmp(arg, "-q")) {
+      nepim_global.soft_error_verbose = 0;
+      continue;
+    }
+
     if (!strcmp(arg, "-f")) {
       int j = i + 1;
 
@@ -337,6 +361,28 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-U")) {
+      ++i;
+      if (i >= argc) {
+        fprintf(stderr, "%s: missing path list\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      nepim_global.unix_list = addr_list_append(nepim_global.unix_list, argv[i]);
+      continue;
+    }
+
+    if (!strcmp(arg, "-G")) {
+      ++i;
+      if (i >= argc) {
+        fprintf(stderr, "%s: missing path list\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      nepim_global.unix_dgram_list = addr_list_append(nepim_global.unix_dgram_list, argv[i]);
+      continue;
+    }
+
     if (!strcmp(arg, "-c")) {
       ++i;
       if (i >= argc) {
