@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: session.h,v 1.13 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: session.h,v 1.14 2006/03/17 23:05:32 evertonm Exp $ */
 
 #ifndef NEPIM_SESSION_H
 #define NEPIM_SESSION_H
@@ -27,6 +27,8 @@ MA 02110-1301 USA
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/time.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 
 #include "greet.h"
 #include "usec.h"
@@ -46,6 +48,7 @@ struct nepim_session_t {
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
+    struct sockaddr_un un;
   }         remote;
   socklen_t remote_len;
 
