@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: slot.c,v 1.29 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: slot.c,v 1.31 2006/03/29 21:23:14 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -231,8 +231,8 @@ int nepim_slot_buf_sendto(nepim_slot_t *slot, char *buf, size_t buf_size, uint8_
   slot_buf_write(slot, buf, buf_size, type);
 
   wr = sendto(slot->udp_sd, buf, buf_size, 0,
-              (const struct sockaddr *) &slot->session.remote,
-              slot->session.remote_len);
+	      (const struct sockaddr *) &slot->session.remote,
+	      slot->session.remote_len);
 
   return wr;
 }
@@ -320,7 +320,7 @@ void nepim_udp_write_keepalive(nepim_slot_t *slot,
     case EAGAIN:
       nepim_usock_write_error(&udp_tab, sd, 
                               slot->index, slot->index_remote,
-                              errno);
+                              errno, nepim_global.soft_error_verbose);
 
       return;
     case EPIPE:
