@@ -18,8 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: sock.c,v 1.26 2005/11/04 09:56:15 evertonm Exp $ */
-
+/* $Id: sock.c,v 1.30 2006/03/31 18:52:53 evertonm Exp $ */
 
 #include <assert.h>
 #include <netdb.h>
@@ -33,6 +32,8 @@ MA 02110-1301 USA
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 
 #include "sock.h"
 
@@ -87,6 +88,7 @@ struct ipv6_mreq {
 #define NEPIM_SOCK_ERR_MCAST_JOIN     (-14)
 #define NEPIM_SOCK_ERR_WIN_RECV       (-15)
 #define NEPIM_SOCK_ERR_WIN_SEND       (-16)
+#define NEPIM_SOCK_ERR_TOS            (-17)
 
 int nepim_sock_get_port(const struct sockaddr *addr)
 {
@@ -95,6 +97,9 @@ int nepim_sock_get_port(const struct sockaddr *addr)
     struct sockaddr_in6 inet6;
   } *sa = (void *) addr;
 
+  if (nepim_sock_family(addr) == AF_UNIX)
+    return -1; /* something obviously invalid */
+
   assert(&(sa->inet.sin_port) == &(sa->inet6.sin6_port));
   assert(sa->inet.sin_port == sa->inet6.sin6_port);
 
@@ -106,28 +111,43 @@ int nepim_sock_family(const struct sockaddr *addr)
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
+    struct sockaddr_un un;
   } *sa = (void *) addr;
 
   assert(&(sa->inet.sin_family) == &(sa->inet6.sin6_family));
   assert(sa->inet.sin_family == sa->inet6.sin6_family);
 
+  assert(&(sa->inet.sin_family) == &(sa->un.sun_family));
+  assert(sa->inet.sin_family == sa->un.sun_family);
+
   return sa->inet.sin_family;
 }
 
-void nepim_sock_dump_addr(char *buf, int buf_size, const struct sockaddr *addr)
+void nepim_sock_dump_addr(char *buf, int buf_size,
+			  const struct sockaddr *addr,
+			  int addr_len)
 {
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
-  } *sa = (void *) addr;
+    struct sockaddr_un un;
+  } *sa;
   const char *dst;
   int family;
 
+  if (addr_len < 1) {
+    assert(buf_size >= 2);
+    snprintf(buf, buf_size, "?");
+    return;
+  }
+
   family = nepim_sock_family(addr);
 
   assert(PF_INET == AF_INET);
   assert(PF_INET6 == AF_INET6);
 
+  sa = (void *) addr;
+
   switch (family) {
   case PF_INET:
     assert(buf_size >= INET_ADDRSTRLEN);
@@ -137,6 +157,10 @@ void nepim_sock_dump_addr(char *buf, int buf_size, const struct sockaddr *addr)
     assert(buf_size >= INET6_ADDRSTRLEN);
     dst = inet_ntop(family, &sa->inet6.sin6_addr, buf, buf_size);
     break;
+  case PF_UNIX:
+    assert(buf_size > strlen(sa->un.sun_path));
+    strcpy(buf, sa->un.sun_path);
+    return; /* skip code below */
   default:
     assert(0);
   }
@@ -245,8 +269,16 @@ static int socket_set_win_send(int sd, int win_send)
   return NEPIM_SOCK_ERR_NONE;
 }
 
+int nepim_socket_tos(int sd, int tos)
+{
+  if (tos < 0)
+    return NEPIM_SOCK_ERR_NONE;
+
+  return setsockopt(sd, SOL_IP, IP_TOS, &tos, sizeof(tos));
+}
+
 static int create_socket(int domain, int type, int protocol, int pmtu_mode, int ttl,
-			 int win_recv, int win_send)
+			 int win_recv, int win_send, int nodelay, int tos)
 {
   int sd;
   int result;
@@ -256,14 +288,14 @@ static int create_socket(int domain, int type, int protocol, int pmtu_mode, int
     return NEPIM_SOCK_ERR_SOCKET;
 
   if (type == SOCK_STREAM) {
-    result = nepim_socket_tcp_opt(sd);
+    result = nepim_socket_tcp_opt(sd, nodelay);
     if (result) {
       close(sd);
       return result;
     }
   }
 
-  result = nepim_socket_opt(sd, pmtu_mode, ttl);
+  result = nepim_socket_opt(sd, pmtu_mode, ttl, tos);
   if (result) {
     close(sd);
     return result;
@@ -335,13 +367,15 @@ int nepim_create_socket(struct sockaddr *addr,
 			int ttl,
 			int mcast_join,
 			int win_recv,
-			int win_send)
+			int win_send,
+			int nodelay,
+			int tos)
 {
   int sd;
   int result;
 
   sd = create_socket(family, type, protocol, pmtu_mode, ttl,
-		     win_recv, win_send);
+		     win_recv, win_send, nodelay, tos);
   if (sd < 0)
     return sd;
 
@@ -350,10 +384,10 @@ int nepim_create_socket(struct sockaddr *addr,
     assert(protocol == IPPROTO_UDP);
     
     if (socket_mcast_join(sd, family, addr, addr_len)) {
-      char buf[INET6_ADDRSTRLEN];
+      char buf[500];
       int e = errno;
 
-      nepim_sock_dump_addr(buf, sizeof(buf), addr);
+      nepim_sock_dump_addr(buf, sizeof(buf), addr, addr_len);
 
       fprintf(stderr, "%s: %s: mcast_join(%d,%s,%d): errno=%d: %s\n",
 	      __FILE__, __PRETTY_FUNCTION__,
@@ -372,15 +406,21 @@ int nepim_create_socket(struct sockaddr *addr,
   }
 
   if (bind(sd, addr, addr_len)) {
-    char buf[INET6_ADDRSTRLEN];
+    char buf[500];
     int e = errno;
 
-    nepim_sock_dump_addr(buf, sizeof(buf), addr);
+    nepim_sock_dump_addr(buf, sizeof(buf), addr, addr_len);
 
-    fprintf(stderr, "%s: %s: bind(%d,%s,%d): errno=%d: %s\n",
-	    __FILE__, __PRETTY_FUNCTION__,
-	    sd, buf, nepim_sock_get_port(addr),
-	    e, strerror(e));
+    if (family == AF_UNIX)
+      fprintf(stderr, "%s: %s: UNIX-socket bind(%d,%s): errno=%d: %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      sd, buf,
+	      e, strerror(e));
+      else
+	fprintf(stderr, "%s: %s: bind(%d,%s,%d): errno=%d: %s\n",
+		__FILE__, __PRETTY_FUNCTION__,
+		sd, buf, nepim_sock_get_port(addr),
+		e, strerror(e));
 
     close(sd);
     return NEPIM_SOCK_ERR_BIND;
@@ -398,13 +438,16 @@ int nepim_create_listener_socket(struct sockaddr *addr,
 				 int pmtu_mode,
 				 int ttl,
 				 int win_recv,
-				 int win_send)
+				 int win_send,
+				 int nodelay,
+				 int tos)
 {
   int sd;
 
   sd = nepim_create_socket(addr, addr_len, family,
 			   type, protocol, pmtu_mode, 
-			   ttl, 0, win_recv, win_send);
+			   ttl, 0, win_recv, win_send,
+			   nodelay, tos);
   if (sd < 0)
     return sd;
 
@@ -440,7 +483,7 @@ static int tcp_nodelay(int sd)
   return setsockopt(sd, SOL_TCP, TCP_NODELAY, &opt, sizeof(opt));
 }
 
-int nepim_socket_opt(int sd, int pmtu_mode, int ttl)
+int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos)
 {
   if (unlinger(sd))
     return NEPIM_SOCK_ERR_UNLINGER;
@@ -454,13 +497,17 @@ int nepim_socket_opt(int sd, int pmtu_mode, int ttl)
   if (nepim_socket_ttl(sd, ttl))
     return NEPIM_SOCK_ERR_TTL;
 
+  if (nepim_socket_tos(sd, tos))
+    return NEPIM_SOCK_ERR_TOS;
+
   return NEPIM_SOCK_ERR_NONE;
 }
 
-int nepim_socket_tcp_opt(int sd)
+int nepim_socket_tcp_opt(int sd, int nodelay)
 {
-  if (tcp_nodelay(sd))
-    return NEPIM_SOCK_ERR_NODELAY;
+  if (nodelay)
+    if (tcp_nodelay(sd))
+      return NEPIM_SOCK_ERR_NODELAY;
 
   return NEPIM_SOCK_ERR_NONE;
 }
@@ -473,13 +520,15 @@ int nepim_connect_client_socket(struct sockaddr *addr,
 				int pmtu_mode,
 				int ttl,
 				int win_recv,
-				int win_send)
+				int win_send,
+				int nodelay,
+				int tos)
 {
   int sd;
   int result;
 
   sd = create_socket(family, type, protocol, pmtu_mode, ttl,
-		     win_recv, win_send);
+		     win_recv, win_send, nodelay, tos);
   if (sd < 0)
     return sd;
 
@@ -501,20 +550,25 @@ int nepim_connect_client_socket(struct sockaddr *addr,
   }
 
   fprintf(stderr, 
-	  "DEBUG FIXME %s %s slow synchronous connect(port=%d)\n",
-	  __FILE__, __PRETTY_FUNCTION__, nepim_sock_get_port(addr));
+	  "DEBUG FIXME %s %s slow synchronous connect\n",
+	  __FILE__, __PRETTY_FUNCTION__);
 
   if (connect(sd, addr, addr_len)) {
-    char buf[INET6_ADDRSTRLEN];
+    char buf[500];
     int e = errno;
 
-    nepim_sock_dump_addr(buf, sizeof(buf), addr);
-
-    fprintf(stderr, "%s: %s: connect(%d,%s,%d): errno=%d: %s\n",
-	    __FILE__, __PRETTY_FUNCTION__,
-	    sd, buf, nepim_sock_get_port(addr),
-	    e, strerror(e));
+    nepim_sock_dump_addr(buf, sizeof(buf), addr, addr_len);
 
+    if (family == PF_UNIX)
+      fprintf(stderr, "%s: %s: UNIX-socket connect(%d,%s): errno=%d: %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      sd, buf, e, strerror(e));
+    else
+      fprintf(stderr, "%s: %s: connect(%d,%s,%d): errno=%d: %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      sd, buf, nepim_sock_get_port(addr),
+	      e, strerror(e));
+    
     close(sd);
     return NEPIM_SOCK_ERR_CONNECT;
   }
@@ -614,6 +668,19 @@ static int socket_get_win_send(int sd)
   return win_send;
 }
 
+int nepim_socket_get_tos(int sd)
+{
+  unsigned char tos;
+  socklen_t optlen = sizeof(tos);
+
+  if (getsockopt(sd, SOL_IP, IP_TOS, &tos, &optlen))
+    return NEPIM_SOCK_ERR_TOS;
+
+  assert(optlen == sizeof(tos));
+
+  return tos;
+}
+
 void nepim_sock_show_opt(FILE *out, int sd)
 {
   int pmtud_mode;
@@ -622,6 +689,7 @@ void nepim_sock_show_opt(FILE *out, int sd)
   int mcast_ttl;
   int win_recv;
   int win_send;
+  int tos;
 
   pmtud_mode = nepim_socket_pmtu_get_mode(sd);
   mtu = nepim_socket_pmtu_get_mtu(sd);
@@ -629,8 +697,9 @@ void nepim_sock_show_opt(FILE *out, int sd)
   mcast_ttl = socket_mcast_get_ttl(sd);
   win_recv = socket_get_win_recv(sd);
   win_send = socket_get_win_send(sd);
+  tos = nepim_socket_get_tos(sd);
 
   fprintf(out, 
-	  "%d: pmtud_mode=%d path_mtu=%d ttl=%d mcast_ttl=%d win_recv=%d win_send=%d\n",
-	  sd, pmtud_mode, mtu, ttl, mcast_ttl, win_recv, win_send);
+	  "%d: pmtud_mode=%d path_mtu=%d tos=%d ttl=%d mcast_ttl=%d win_recv=%d win_send=%d\n",
+	  sd, pmtud_mode, mtu, tos, ttl, mcast_ttl, win_recv, win_send);
 }
