@@ -29,12 +29,13 @@ int nepim_socket_block(int sd);
 int nepim_socket_ttl(int sd, int ttl);
 int nepim_socket_mcast_ttl(int sd, int mcast_ttl);
 int nepim_socket_pmtu(int sd, int pmtu_mode);
-int nepim_socket_opt(int sd, int pmtu_mode, int ttl);
-int nepim_socket_tcp_opt(int sd);
+int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos);
+int nepim_socket_tcp_opt(int sd, int nodelay);
 
 int nepim_socket_pmtu_get_mode(int sd);
 int nepim_socket_pmtu_get_mtu(int sd);
 int nepim_socket_get_ttl(int sd);
+int nepim_socket_get_tos(int sd);
 
 int nepim_create_socket(struct sockaddr *addr,
 			int addr_len,
@@ -45,7 +46,9 @@ int nepim_create_socket(struct sockaddr *addr,
 			int ttl,
 			int mcast_join,
 			int win_recv,
-			int win_send);
+			int win_send,
+			int nodelay,
+			int tos);
 int nepim_create_listener_socket(struct sockaddr *addr,
 				 int addr_len,
 				 int family,
@@ -55,7 +58,9 @@ int nepim_create_listener_socket(struct sockaddr *addr,
 				 int pmtu_mode,
 				 int ttl,
 				 int win_recv,
-				 int win_send);
+				 int win_send,
+				 int nodelay,
+				 int tos);
 int nepim_connect_client_socket(struct sockaddr *addr,
 				int addr_len,
 				int family,
@@ -64,10 +69,14 @@ int nepim_connect_client_socket(struct sockaddr *addr,
 				int pmtu_mode,
 				int ttl,
 				int win_recv,
-				int win_send);
+				int win_send,
+				int nodelay,
+				int tos);
 
 int nepim_sock_get_port(const struct sockaddr *addr);
-void nepim_sock_dump_addr(char *buf, int size, const struct sockaddr *addr);
+void nepim_sock_dump_addr(char *buf, int buf_size,
+			  const struct sockaddr *addr,
+			  int addr_len);
 int nepim_sock_family(const struct sockaddr *addr);
 void nepim_sock_show_opt(FILE *out, int sd);
 
