@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_client.c,v 1.47 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.51 2006/03/30 23:36:49 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -29,6 +29,9 @@ MA 02110-1301 USA
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
+#include <sys/un.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include <assert.h>
 
 #include "conf.h"
@@ -221,6 +224,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
+    struct sockaddr_un un;
   } from;
   socklen_t fromlen = sizeof(from);
   nepim_udp_hdr_t hdr;
@@ -236,7 +240,8 @@ static void *on_udp_read_segment(oop_source *src, int sd,
     case EINTR:
     case EAGAIN:
     case ECONNREFUSED:
-      nepim_usock_read_error(&udp_tab, sd, errno);
+      nepim_usock_read_error(&udp_tab, sd, errno,
+			     nepim_global.soft_error_verbose);
       return OOP_CONTINUE;
     }
 
@@ -386,7 +391,7 @@ static int slot_greet_write(nepim_slot_t *slot)
     case ECONNREFUSED:
       nepim_usock_write_error(&udp_tab, slot->udp_sd, 
                               slot->index, slot->index_remote,
-                              errno);
+                              errno, nepim_global.soft_error_verbose);
       return 0; /* ignore as soft error (caller always resend) */
     default:
       fprintf(stderr, 
@@ -652,7 +657,7 @@ static void udp_write_rate(nepim_slot_t *slot)
     case ECONNREFUSED:
       nepim_usock_write_error(&udp_tab, sd, 
                               slot->index, slot->index_remote,
-                              errno);
+                              errno, nepim_global.soft_error_verbose);
       return;
 
     case EPIPE:
@@ -731,7 +736,7 @@ static void udp_write_full(nepim_slot_t *slot)
     case ECONNREFUSED:
       nepim_usock_write_error(&udp_tab, sd, 
                               slot->index, slot->index_remote,
-                              errno);
+                              errno, nepim_global.soft_error_verbose);
       return;
 
     case EPIPE:
@@ -1058,38 +1063,88 @@ static int spawn_udp_connection(const char *hostname,
   struct addrinfo hints;
   struct addrinfo *ai_res;
   struct addrinfo *ai;
+  struct sockaddr_un un_sock_addr;
+  struct stat stat_buf;
+  int gai_lookup; /* boolean */
   int result;
   int sd = -1;
 
   memset(&hints, 0, sizeof(hints));
 
-  hints.ai_socktype = SOCK_DGRAM;
-  hints.ai_protocol = IPPROTO_UDP;
-  hints.ai_flags = AI_CANONNAME;
-  hints.ai_family = PF_UNSPEC;
-  hints.ai_addrlen = 0;
-  hints.ai_addr = 0;
-  hints.ai_canonname = 0;
+  if (nepim_global.search_unix_socket) {
+    gai_lookup = stat(hostname, &stat_buf);
+    if (gai_lookup)
+      fprintf(stderr, "not a UNIX domain path: %s: errno=%d: %s\n",
+	      hostname, errno, strerror(errno));
+  }
+  else
+    gai_lookup = -1;
 
-  fprintf(stderr, 
-          "UDP socket solving %s,%s\n",
-          hostname, portname);
+  if (gai_lookup) {
+    /*
+      Lookup hostname
+    */
+    
+    hints.ai_socktype = SOCK_DGRAM;
+    hints.ai_protocol = IPPROTO_UDP;
+    hints.ai_flags = AI_CANONNAME;
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_addrlen = 0;
+    hints.ai_addr = 0;
+    hints.ai_canonname = 0;
 
-  result = getaddrinfo(hostname, portname,
-                       &hints, &ai_res);
-  if (result) {
-    fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
-            __PRETTY_FUNCTION__, 
-            hostname, portname,
-            gai_strerror(result));
-    return -1;
+    fprintf(stderr, 
+	    "UDP socket solving %s,%s\n",
+	    hostname, portname);
+
+    result = getaddrinfo(hostname, portname,
+			 &hints, &ai_res);
+    if (result) {
+      fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
+	      __PRETTY_FUNCTION__, 
+	      hostname, portname,
+	      gai_strerror(result));
+      return -1;
+    }
+  }
+  else {
+    /*
+      Unix socket
+    */
+    
+    int len = strlen(hostname);
+    if (len >= (int) sizeof(un_sock_addr.sun_path)) {
+      fprintf(stderr, "%s %s: unix socket path overly long: len=%d > max=%d\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      len, sizeof(un_sock_addr.sun_path) - 1);
+      return -1;
+    }
+
+    memset(&un_sock_addr, 0, sizeof(un_sock_addr));
+
+    un_sock_addr.sun_family = AF_UNIX;
+    strcpy(un_sock_addr.sun_path, hostname);
+
+    hints.ai_socktype = SOCK_DGRAM;
+    hints.ai_protocol = 0;
+    hints.ai_family = AF_UNIX;
+    hints.ai_addrlen = sizeof(un_sock_addr);
+    hints.ai_addr = (struct sockaddr *) &un_sock_addr;
+    hints.ai_canonname = 0;
+
+    ai_res = &hints;
   }
 
   for (ai = ai_res; ai; ai = ai->ai_next) {
 
-    fprintf(stderr, 
-            "UDP socket trying %s,%s\n",
-            hostname, portname);
+    if (ai->ai_family == PF_UNIX)
+      fprintf(stderr, 
+	      "UNIX datagram socket trying path=%s\n",
+	      hostname);
+    else
+      fprintf(stderr, 
+	      "UDP socket trying %s,%s\n",
+	      hostname, portname);
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
       continue;
@@ -1103,11 +1158,18 @@ static int spawn_udp_connection(const char *hostname,
                                      nepim_global.pmtu_mode,
                                      nepim_global.ttl,
                                      nepim_global.win_recv,
-                                     nepim_global.win_send);
+                                     nepim_global.win_send,
+				     0 /* tcp nodelay */,
+				     nepim_global.tos);
     if (sd < 0) {
-      fprintf(stderr, 
-              "could not connect UDP socket to %s,%s: %d\n",
-              hostname, portname, sd);
+      if (ai->ai_family == AF_UNIX)
+	fprintf(stderr, 
+		"could not connect UNIX datagram socket to path=%s: %d\n",
+		hostname, sd);
+      else
+	fprintf(stderr, 
+		"could not connect UDP socket to %s,%s: %d\n",
+		hostname, portname, sd);
       continue;
     }
 
@@ -1127,34 +1189,42 @@ static int spawn_udp_connection(const char *hostname,
       union {
         struct sockaddr_in inet;
         struct sockaddr_in6 inet6;
+	struct sockaddr_un un;
       } local_sockaddr;
       socklen_t local_sockaddr_len = sizeof(local_sockaddr);
       int result;
-      char local[100];
+      char local[500];
       
       result = getsockname(sd, (struct sockaddr *) &local_sockaddr,
                            &local_sockaddr_len);
       assert(!result);
 
       nepim_sock_dump_addr(local, sizeof(local),
-                           (const struct sockaddr *) &local_sockaddr);
+                           (const struct sockaddr *) &local_sockaddr,
+			   local_sockaddr_len);
 
-      fprintf(stderr, 
-              "%d: UDP socket (%s,%d) connected to: "
-              "host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
-              sd, 
-              local,
-              nepim_sock_get_port((struct sockaddr *) &local_sockaddr),
-              hostname, portname, 
-              nepim_sock_get_port(ai->ai_addr),
-              ai->ai_addrlen, ai->ai_family, 
-              ai->ai_socktype, ai->ai_protocol);
+      if (ai->ai_family == AF_UNIX)
+	fprintf(stderr, 
+		"%d: UNIX datagram socket connected to: path=%s",
+		sd, local);
+      else
+	  fprintf(stderr, 
+		  "%d: UDP socket (%s,%d) connected to: "
+		  "host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
+		  sd, 
+		  local,
+		  nepim_sock_get_port((struct sockaddr *) &local_sockaddr),
+		  hostname, portname, 
+		  nepim_sock_get_port(ai->ai_addr),
+		  ai->ai_addrlen, ai->ai_family, 
+		  ai->ai_socktype, ai->ai_protocol);
     }
 
     break;
   }
 
-  freeaddrinfo(ai_res);
+  if (gai_lookup)
+    freeaddrinfo(ai_res);
 
   return sd;
 }
@@ -1186,6 +1256,7 @@ static void parse_hosts(const char *host_list)
       union {
         struct sockaddr_in inet;
         struct sockaddr_in6 inet6;
+	struct sockaddr_un un;
       } remote;
       int remote_len = sizeof(remote);
       int sd;
