@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_server.c,v 1.71 2005/12/22 09:41:07 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.75 2006/03/30 23:36:49 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -420,7 +420,7 @@ static void udp_write_rate(nepim_slot_t *slot)
     case EAGAIN:
       nepim_usock_write_error(&udp_tab, sd, 
 			      slot->index, slot->index_remote,
-			      errno);
+			      errno, nepim_global.soft_error_verbose);
 
       return;
     case EPIPE:
@@ -488,12 +488,15 @@ static void udp_write_full(nepim_slot_t *slot)
     case EAGAIN:
       nepim_usock_write_error(&udp_tab, sd, 
 			      slot->index, slot->index_remote,
-			      errno);
+			      errno, nepim_global.soft_error_verbose);
 
       return;
     case EPIPE:
       fprintf(stderr, "write: EPIPE on UDP socket %d\n", sd);
       break;
+    default:
+      fprintf(stderr, "write: UDP socket: sd=%d errno=%d: %s\n",
+	      sd, errno, strerror(errno));
     }
 
     fprintf(stderr, "write: connection lost on UDP socket %d\n", sd);
@@ -696,7 +699,7 @@ static int parse_greetings(nepim_greet_t *opt,
   int result;
   char *eos;
   char *past_end;
-  char peer[100];
+  char peer[500];
   char password_buf[100];
 
   eos = memchr(buf, '\n', buf_len);
@@ -735,7 +738,8 @@ static int parse_greetings(nepim_greet_t *opt,
     return result;
 
   nepim_sock_dump_addr(peer, sizeof(peer),
-		       (const struct sockaddr *) remote);
+		       (const struct sockaddr *) remote,
+		       remote_len);
 
   fprintf(stderr, 
 	  "%d %d-%d: UDP from (%s,%d): send=%d bps=%lld pps=%d "
@@ -799,6 +803,7 @@ static void *on_udp_segment(oop_source *src, int sd,
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
+    struct sockaddr_un un;
   } from;
   socklen_t fromlen = sizeof(from);
   nepim_udp_hdr_t hdr;
@@ -814,7 +819,8 @@ static void *on_udp_segment(oop_source *src, int sd,
     case EINTR:
     case EAGAIN:
     case ECONNREFUSED:
-      nepim_usock_read_error(&udp_tab, sd, errno);
+      nepim_usock_read_error(&udp_tab, sd, errno,
+			     nepim_global.soft_error_verbose);
       return OOP_CONTINUE;
     }
 
@@ -827,9 +833,24 @@ static void *on_udp_segment(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+
   assert(rd >= 0);
   assert(rd <= sizeof(buf));
 
+  /*
+    For UNIX-domain datagram sockets.
+   */
+  if (fromlen < 1) {
+    fromlen = sizeof(from);
+    if (getsockname(sd, (struct sockaddr *) &from, &fromlen)) {
+      fprintf(stderr,
+	      "%d: getsockname: errno=%d: %s\n",
+	      sd, errno, strerror(errno));
+    }
+  }
+
+  assert(fromlen > 0);
+
   nepim_usock_read_good(&udp_tab, sd);
 
   if (nepim_udp_hdr_parse(&hdr, buf, rd)) {
@@ -1015,10 +1036,12 @@ int nepim_udp_listener(const char *hostname, const char *portname,
 			     nepim_global.ttl,
 			     mcast_join,
 			     nepim_global.win_recv,
-			     nepim_global.win_send);
+			     nepim_global.win_send,
+			     0 /* tcp nodelay */,
+			     nepim_global.tos);
     if (sd < 0) {
       fprintf(stderr, 
-	      "spawn_udp_listener: UDP listener socket failed for %s,%s(%d): %d\n",
+	      "nepim_udp_listener: UDP listener socket failed for %s,%s(%d): %d\n",
 	      hostname, portname, nepim_sock_get_port(ai->ai_addr), sd);
       break;
     }
@@ -1042,3 +1065,56 @@ int nepim_udp_listener(const char *hostname, const char *portname,
 
   return udp_listeners;
 }
+
+int nepim_unix_dgram_listener(const char *path)
+{
+  struct sockaddr_un un_sock_addr;
+  int len;
+  int sd;
+
+  len = strlen(path);
+  if (len >= (int) sizeof(un_sock_addr.sun_path)) {
+    fprintf(stderr, "%s %s: unix socket path overly long: len=%d > max=%d\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    len, sizeof(un_sock_addr.sun_path) - 1);
+    return 0;
+  }
+  
+  memset(&un_sock_addr, 0, sizeof(un_sock_addr));
+  
+  un_sock_addr.sun_family = AF_UNIX;
+  strcpy(un_sock_addr.sun_path, path);
+  
+  sd = nepim_create_socket((struct sockaddr *) &un_sock_addr, 
+			   sizeof(un_sock_addr),
+			   AF_UNIX,
+			   SOCK_DGRAM, 
+			   0,
+			   nepim_global.pmtu_mode,
+			   nepim_global.ttl,
+			   0 /* mcast_join */,
+			   nepim_global.win_recv,
+			   nepim_global.win_send,
+			   0 /* tcp nodelay */,
+			   nepim_global.tos);
+  if (sd < 0) {
+    fprintf(stderr, 
+	    "%s %s: UNIX datagram listener socket failed for path=%s\n",
+	    __FILE__, __PRETTY_FUNCTION__, path); 
+    return 0;
+  }
+
+  nepim_global.oop_src->on_fd(nepim_global.oop_src,
+			      sd, OOP_READ,
+			      on_udp_segment, 0);
+
+  nepim_usock_set_add(&udp_tab, sd);
+
+  fprintf(stderr, 
+	  "%d: UNIX datagram socket listening on path=%s\n",
+	  sd, path);
+
+  nepim_sock_show_opt(stderr, sd);
+
+  return 1;
+}
